@@ -42,12 +42,19 @@ class YOLOv4(Model):
         xyscales,
         activation0: str = "mish",
         activation1: str = "leaky",
+        kernel_regularizer=None,
     ):
         super(YOLOv4, self).__init__(name="YOLOv4")
         self.csp_darknet53 = CSPDarknet53(
-            activation0=activation0, activation1=activation1
+            activation0=activation0,
+            activation1=activation1,
+            kernel_regularizer=kernel_regularizer,
+        )
+        self.panet = PANet(
+            num_classes=num_classes,
+            activation=activation1,
+            kernel_regularizer=kernel_regularizer,
         )
-        self.panet = PANet(num_classes=num_classes, activation=activation1)
         self.yolov3_head = YOLOv3Head(
             anchors=anchors, num_classes=num_classes, xysclaes=xyscales
         )
@@ -99,15 +106,19 @@ class YOLOv4Tiny(Model):
         num_classes: int,
         xyscales,
         activation: str = "leaky",
-        tpu: bool = False,
+        kernel_regularizer=None,
     ):
         super(YOLOv4Tiny, self).__init__(name="YOLOv4Tiny")
-        self.csp_darknet53_tiny = CSPDarknet53Tiny(activation=activation)
+        self.csp_darknet53_tiny = CSPDarknet53Tiny(
+            activation=activation, kernel_regularizer=kernel_regularizer,
+        )
         self.panet_tiny = PANetTiny(
-            num_classes=num_classes, activation=activation
+            num_classes=num_classes,
+            activation=activation,
+            kernel_regularizer=kernel_regularizer,
         )
         self.yolov3_head_tiny = YOLOv3HeadTiny(
-            anchors=anchors, num_classes=num_classes, xysclaes=xyscales, tpu=tpu
+            anchors=anchors, num_classes=num_classes, xysclaes=xyscales
         )
 
     def call(self, x):
