@@ -46,13 +46,15 @@ class YOLOv4(BaseClass):
         super(YOLOv4, self).__init__(tiny=tiny, tpu=tpu)
 
         self.batch_size = 32
-        self.subdivision = 16
         self._has_weights = False
         self.input_size = 608
         self.model = None
 
     def make_model(
-        self, activation0: str = "mish", activation1: str = "leaky",
+        self,
+        activation0: str = "mish",
+        activation1: str = "leaky",
+        kernel_regularizer=tf.keras.regularizers.l2(0.0005),
     ):
         # pylint: disable=missing-function-docstring
         self._has_weights = False
@@ -65,7 +67,7 @@ class YOLOv4(BaseClass):
                 num_classes=len(self.classes),
                 xyscales=self.xyscales,
                 activation=activation1,
-                tpu=self.tpu,
+                kernel_regularizer=kernel_regularizer,
             )
         else:
             self.model = yolov4.YOLOv4(
@@ -74,6 +76,7 @@ class YOLOv4(BaseClass):
                 xyscales=self.xyscales,
                 activation0=activation0,
                 activation1=activation1,
+                kernel_regularizer=kernel_regularizer,
             )
         self.model(inputs)
 
@@ -188,79 +191,44 @@ class YOLOv4(BaseClass):
         pred_bboxes = self.fit_pred_bboxes_to_original(pred_bboxes, frame.shape)
         return pred_bboxes
 
-    def inference(self, media_path, is_image=True, cv_waitKey_delay=10):
-        if not path.exists(media_path):
-            raise FileNotFoundError("{} does not exist".format(media_path))
-        if is_image:
-            frame = cv2.imread(media_path)
-            frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
-
-            start_time = time.time()
-            bboxes = self.predict(frame)
-            exec_time = time.time() - start_time
-            print("time: {:.2f} ms".format(exec_time * 1000))
-
-            frame = cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)
-            image = self.draw_bboxes(frame, bboxes)
-            cv2.namedWindow("result", cv2.WINDOW_AUTOSIZE)
-            cv2.imshow("result", image)
-        else:
-            vid = cv2.VideoCapture(media_path)
-            while True:
-                return_value, frame = vid.read()
-                if return_value:
-                    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
-                else:
-                    break
-
-                start_time = time.time()
-                bboxes = self.predict(frame)
-                curr_time = time.time()
-                exec_time = curr_time - start_time
-                info = "time: %.2f ms" % (1000 * exec_time)
-                print(info)
-
-                frame = cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)
-                image = self.draw_bboxes(frame, bboxes)
-                cv2.namedWindow("result", cv2.WINDOW_AUTOSIZE)
-                cv2.imshow("result", image)
-                if cv2.waitKey(cv_waitKey_delay) & 0xFF == ord("q"):
-                    break
-
-        print("YOLOv4: Inference is finished")
-        while cv2.waitKey(10) & 0xFF != ord("q"):
-            pass
-        cv2.destroyWindow("result")
-
     ############
     # Training #
     ############
 
     def load_dataset(
-        self, dataset_path, dataset_type="converted_coco", training=True
+        self,
+        dataset_path,
+        dataset_type="converted_coco",
+        label_smoothing=0.1,
+        image_path_prefix=None,
+        training=True,
     ):
         return dataset.Dataset(
             anchors=self.anchors,
-            batch_size=self.batch_size // self.subdivision,
+            batch_size=self.batch_size,
             dataset_path=dataset_path,
             dataset_type=dataset_type,
             data_augmentation=training,
             input_size=self.input_size,
+            label_smoothing=label_smoothing,
             num_classes=len(self.classes),
+            image_path_prefix=image_path_prefix,
             strides=self.strides,
             xyscales=self.xyscales,
         )
 
     def compile(
         self,
-        optimizer=optimizers.Adam(learning_rate=1e-4),
         loss_iou_type: str = "ciou",
+        loss_verbose=1,
+        optimizer=optimizers.Adam(learning_rate=1e-4),
     ):
         self.model.compile(
             optimizer=optimizer,
             loss=train.YOLOv4Loss(
-                batch_size=self.batch_size // self.subdivision,
+                batch_size=self.batch_size,
                 iou_type=loss_iou_type,
+                verbose=loss_verbose,
             ),
         )
 
@@ -268,16 +236,17 @@ class YOLOv4(BaseClass):
         self,
         data_set,
         epochs,
-        verbose=1,
+        verbose=2,
         callbacks=None,
         validation_data=None,
         initial_epoch=0,
+        steps_per_epoch=None,
         validation_steps=None,
         validation_freq=1,
     ):
         self.model.fit(
             data_set,
-            batch_size=self.batch_size // self.subdivision,
+            batch_size=self.batch_size,
             epochs=epochs,
             verbose=verbose,
             callbacks=callbacks,
@@ -287,7 +256,7 @@ class YOLOv4(BaseClass):
             class_weight=None,
             sample_weight=None,
             initial_epoch=initial_epoch,
-            steps_per_epoch=self.subdivision,
+            steps_per_epoch=steps_per_epoch,
             validation_steps=validation_steps,
             validation_batch_size=None,
             validation_freq=validation_freq,
@@ -328,10 +297,9 @@ class YOLOv4(BaseClass):
             image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
             height, width, _ = image.shape
 
-            if data_set.dataset_type == "yolo":
-                _dataset[1] = _dataset[1] * np.array(
-                    [width, height, width, height, 1]
-                )
+            _dataset[1] = _dataset[1] * np.array(
+                [width, height, width, height, 1]
+            )
 
             # ground-truth
             with open(
