@@ -26,12 +26,13 @@ from os import makedirs, path
 import tensorflow as tf
 from tensorflow.keras import backend
 from tensorflow.keras.callbacks import Callback
-from tensorflow.keras.losses import Loss
+from tensorflow.keras.losses import BinaryCrossentropy, Loss, Reduction
 
 
 class YOLOv4Loss(Loss):
-    def __init__(self, batch_size, iou_type):
+    def __init__(self, batch_size, iou_type, verbose=0):
         super(YOLOv4Loss, self).__init__(name="YOLOv4Loss")
+        self.batch_size = batch_size
         if iou_type == "iou":
             self.bbox_xiou = bbox_iou
         elif iou_type == "giou":
@@ -39,9 +40,14 @@ class YOLOv4Loss(Loss):
         elif iou_type == "ciou":
             self.bbox_xiou = bbox_ciou
 
-        self.batch_size = batch_size
+        self.verbose = verbose
+
         self.while_cond = lambda i, iou: tf.less(i, self.batch_size)
 
+        self.prob_binaryCrossentropy = BinaryCrossentropy(
+            reduction=Reduction.NONE
+        )
+
     def call(self, y_true, y_pred):
         """
         @param y_true: Dim(batch, grid, grid, 3,
@@ -66,6 +72,8 @@ class YOLOv4Loss(Loss):
         truth_conf = y_true[..., 4:5]
         truth_prob = y_true[..., 5:]
 
+        num_classes = truth_prob.shape[-1]
+
         pred_xywh = y_pred[..., 0:4]
         pred_conf = y_pred[..., 4:5]
         pred_prob = y_pred[..., 5:]
@@ -74,7 +82,7 @@ class YOLOv4Loss(Loss):
         num_obj = tf.reduce_sum(one_obj, axis=[1, 2])
         one_noobj = 1.0 - one_obj
         # Dim(batch, grid * grid * 3, 1)
-        one_obj_mask = one_obj > 0.9
+        one_obj_mask = one_obj > 0.5
 
         zero = tf.zeros((1, grid_size * grid_size * 3, 1), dtype=tf.float32)
 
@@ -82,6 +90,7 @@ class YOLOv4Loss(Loss):
         xiou = self.bbox_xiou(truth_xywh, pred_xywh)
         xiou_scale = 2.0 - truth_xywh[..., 2:3] * truth_xywh[..., 3:4]
         xiou_loss = one_obj * xiou_scale * (1.0 - xiou[..., tf.newaxis])
+        xiou_loss = 3 * tf.reduce_mean(tf.reduce_sum(xiou_loss, axis=(1, 2)))
 
         # Confidence Loss
         i0 = tf.constant(0)
@@ -130,23 +139,34 @@ class YOLOv4Loss(Loss):
             * tf.cast(max_iou < 0.5, dtype=tf.float32)
             * (0.0 - backend.log(1.0 - pred_conf + 1e-9))
         )
-
-        # Probabilities Loss
-        prob_pos_loss = truth_prob * (0 - backend.log(pred_prob + 1e-9))
-        prob_neg_loss = (1 - truth_prob) * (
-            0 - backend.log(1 - pred_prob + 1e-9)
-        )
-        prob_loss = one_obj * tf.reduce_sum(
-            prob_pos_loss + prob_neg_loss, axis=-1, keepdims=True
-        )
-
-        sum_xiou_loss = tf.reduce_mean(tf.reduce_sum(xiou_loss, axis=(1, 2)))
-        sum_conf_loss = tf.reduce_mean(
+        conf_loss = tf.reduce_mean(
             tf.reduce_sum(conf_obj_loss + conf_noobj_loss, axis=(1, 2))
         )
-        sum_prob_loss = tf.reduce_mean(tf.reduce_sum(prob_loss, axis=(1, 2)))
 
-        return sum_xiou_loss + sum_conf_loss + sum_prob_loss
+        # Probabilities Loss
+        prob_loss = self.prob_binaryCrossentropy(truth_prob, pred_prob)
+        prob_loss = one_obj * prob_loss[..., tf.newaxis]
+        prob_loss = tf.reduce_mean(
+            tf.reduce_sum(prob_loss, axis=(1, 2)) * num_classes
+        )
+
+        total_loss = xiou_loss + conf_loss + prob_loss
+
+        if self.verbose != 0:
+            tf.print(
+                "grid:",
+                grid_size,
+                "iou_loss:",
+                xiou_loss,
+                "conf_loss:",
+                conf_loss,
+                "prob_loss:",
+                prob_loss,
+                "total_loss",
+                total_loss,
+            )
+
+        return total_loss
 
 
 def bbox_iou(bboxes1, bboxes2):
